@@ -27,30 +27,40 @@
 # ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 # POSSIBILITY OF SUCH DAMAGE.
 
-try:
-    import json
-except ImportError:
-    import simplejson as json
-
+import cgi
 from datetime import datetime
+try:
+    from email.feedparser import FeedParser
+    from email.header import Header
+except ImportError:
+    from email.Parser import FeedParser
+    from email.Header import Header
 import logging
+import os
 import random
+import re
+from StringIO import StringIO
 import sys
 import traceback
 import unittest
+from urlparse import urlparse
 
 import mox
+from oauth.oauth import OAuthConsumer, OAuthToken
+import simplejson as json
 
 import typepad
 from tests import utils
 
 
-def json_equals(data):
+def json_equals(data, text):
+    otherdata = json.loads(text)
+    return bool(data == otherdata)
+
+
+def json_equals_func(data):
     def confirm_equals_data(text):
-        otherdata = json.loads(text)
-        if data == otherdata:
-            return True
-        return False
+        return json_equals(data, text)
     return confirm_equals_data
 
 
@@ -108,7 +118,10 @@ requests = {
     ),
     'create_post': (
         { 'uri': 'http://127.0.0.1:8000/groups/1/post-assets.json',
-          'headers': {'accept': 'application/json'},
+          'headers': {
+              'accept': 'application/json',
+              'content-type': 'application/json',
+          },
           'body': """{"content": "Hi this post has some content is it not nifty", "objectTypes": ["tag:api.typepad.com,2009:Post"], "categories": [{"term": "fred"}, {"term": "wilma"}], "title": "New post #47"}""",
           'method': 'POST' },
         { 'status': 201,
@@ -159,8 +172,9 @@ requests = {
           'headers': {
             'if-match': '7',
             'accept': 'application/json',
+            'content-type': 'application/json',
           },
-          'body': mox.Func(json_equals({"content": "Yay this is my post", "objectTypes": ["tag:api.typepad.com,2009:Post"], "title": "Omg hai"})),
+          'body': mox.Func(json_equals_func({"content": "Yay this is my post", "objectTypes": ["tag:api.typepad.com,2009:Post"], "title": "Omg hai"})),
           'method': 'PUT' },
         { 'content': """{"content": "Yay this is my post", "objectTypes": ["tag:api.typepad.com,2009:Post"], "title": "Omg hai"}""",
           'etag': 'xyz' },
@@ -189,19 +203,19 @@ class TestAsset(unittest.TestCase):
         typepad.client = self.typepad_client
         del self.typepad_client
 
-    def testUser(self):
+    def test_user(self):
         h = self.http('get_user')
         user = typepad.User.get('http://127.0.0.1:8000/users/1.json', http=h)
         self.assertEquals(user.display_name, 'Deanna Conti')
         mox.Verify(h)
 
-    def testGroup(self):
+    def test_group(self):
         h = self.http('get_group')
         group = typepad.Group.get('http://127.0.0.1:8000/groups/1.json', http=h)
         self.assertEquals(group.display_name, 'Augue Tempor')
         mox.Verify(h)
 
-    def testGroupMembers(self):
+    def test_group_members(self):
         g = typepad.Group()
         g._location = 'http://127.0.0.1:8000/groups/1.json'
 
@@ -229,7 +243,7 @@ class TestAsset(unittest.TestCase):
         m.deliver()
         mox.Verify(h)
 
-    def testCreateDeletePost(self):
+    def test_create_delete_post(self):
         g = typepad.Group.get('http://127.0.0.1:8000/groups/1.json')
 
         p = typepad.Post(
@@ -268,7 +282,7 @@ class TestAsset(unittest.TestCase):
         self.assertRaises(typepad.Post.NotFound, lambda: not_there.title)
         mox.Verify(h)
 
-    def testChangePost(self):
+    def test_change_post(self):
         # Get post #1 directly from group #1?
         g = typepad.Group.get('http://127.0.0.1:8000/groups/1.json')
 
@@ -293,7 +307,7 @@ class TestAsset(unittest.TestCase):
 
 class TestLocally(unittest.TestCase):
 
-    def checkClassyAssets(self, make_asset):
+    def check_classy_assets(self, make_asset):
         data = {
             'id':          '7',
             'urlId':       '7',
@@ -320,12 +334,10 @@ class TestLocally(unittest.TestCase):
         a = make_asset(data)
         self.assertEquals(type(a), typepad.Post)
 
+    def test_classy_assets_from_dict(self):
+        self.check_classy_assets(make_asset=typepad.Asset.from_dict)
 
-    def testClassyAssetsFromDict(self):
-        self.checkClassyAssets(make_asset=typepad.Asset.from_dict)
-
-
-    def testClassyAssetObjectFields(self):
+    def test_classy_asset_object_fields(self):
         class AssetKeeper(typepad.TypePadObject):
             asset = typepad.fields.Object('Asset')
 
@@ -333,36 +345,63 @@ class TestLocally(unittest.TestCase):
             k = AssetKeeper.from_dict({'asset': data})
             return k.asset
 
-        self.checkClassyAssets(make_asset)
+        self.check_classy_assets(make_asset)
 
+    def test_unclassy_assetrefs(self):
+        data = {
+            'id':          '7',
+            'urlId':       '7',
+            'title':       'some asset',
+            'objectTypes': ('http://example.com/internet/',),
+        }
 
-    def testGroupMembershipWithTimestamps(self):
+        class AssetRefKeeper(typepad.TypePadObject):
+            assetref = typepad.fields.Object('AssetRef')
+
+        ar = typepad.AssetRef.from_dict(data)
+        self.assert_(isinstance(ar, typepad.AssetRef))
+        ar = AssetRefKeeper.from_dict({'assetref': data}).assetref
+        self.assert_(isinstance(ar, typepad.AssetRef))
+
+        data['objectTypes'] = ('tag:api.typepad.com,2009:Post',)
+
+        # Even with a known object type, AssetRefs stay AssetRefs.
+        ar = typepad.AssetRef.from_dict(data)
+        self.assert_(isinstance(ar, typepad.AssetRef))
+        ar = AssetRefKeeper.from_dict({'assetref': data}).assetref
+        self.assert_(isinstance(ar, typepad.AssetRef))
+
+    def test_group_membership_with_timestamps(self):
         data = {
             'created': {
                 'tag:api.typepad.com,2009:Admin':  '2009-01-07T00:00:00Z',
                 'tag:api.typepad.com,2009:Member': '2009-01-03T00:00:00Z',
             },
-            'types': ('tag:api.typepad.com,2009:Member',
-                      'tag:api.typepad.com,2009:Admin'),
+            'status': {
+                'types': ('tag:api.typepad.com,2009:Member',
+                          'tag:api.typepad.com,2009:Admin')
+            }
         }
 
-        r = typepad.RelationshipStatus.from_dict(data)
-        self.assertEquals(len(r.types), 2)
+        r = typepad.Relationship.from_dict(data)
+        self.assertEquals(len(r.status.types), 2)
 
-        types = r.types
-        self.assert_(types[0].uri)
+        types = r.status.types
+        self.assert_(types[0])
+
+        created = r.created
+        self.assert_(created['tag:api.typepad.com,2009:Member'])
 
         # Put the list in an expected order: Admin first.
-        if types[0].uri != 'tag:api.typepad.com,2009:Admin':
+        if types[0] != 'tag:api.typepad.com,2009:Admin':
             types.reverse()
 
-        self.assertEquals(types[0].uri, 'tag:api.typepad.com,2009:Admin')
-        self.assertEquals(types[0].created, datetime(2009, 1, 7, 0, 0, 0))
-        self.assertEquals(types[1].uri, 'tag:api.typepad.com,2009:Member')
-        self.assertEquals(types[1].created, datetime(2009, 1, 3, 0, 0, 0))
+        self.assertEquals(types[0], 'tag:api.typepad.com,2009:Admin')
+        self.assertEquals(created[types[0]], datetime(2009, 1, 7, 0, 0, 0))
+        self.assertEquals(types[1], 'tag:api.typepad.com,2009:Member')
+        self.assertEquals(created[types[1]], datetime(2009, 1, 3, 0, 0, 0))
 
-
-    def testRelationships(self):
+    def test_relationships(self):
         data = {
             'source': {
                 'displayName': 'Nikola',
@@ -407,7 +446,7 @@ class TestLocally(unittest.TestCase):
         self.assert_(isinstance(r.source, typepad.Group))
         self.assert_(isinstance(r.target, typepad.User))
 
-    def testUserUrlId(self):
+    def test_user_url_id(self):
         def valid(v):
             self.assert_(typepad.User.get_by_url_id(v))
 
@@ -431,6 +470,230 @@ class TestLocally(unittest.TestCase):
         invalid('^hat^hat^')
 
 
+class TestBrowserUpload(unittest.TestCase):
+
+    def setUp(self):
+        self.typepad_client = typepad.client
+
+    def tearDown(self):
+        typepad.client = self.typepad_client
+        del self.typepad_client
+
+        for x in ('headers', 'body'):
+            try:
+                delattr(self, x)
+            except AttributeError:
+                pass
+
+    def saver(self, fld):
+        def save_data(data):
+            setattr(self, fld, data)
+            return True
+        return save_data
+
+    def message_from_response(self, headers, body):
+        fp = FeedParser()
+        for header, value in headers.iteritems():
+            fp.feed("%s: %s\n" % (header, Header(value).encode()))
+        fp.feed("\n")
+        fp.feed(body)
+        response = fp.close()
+        return response
+
+    def test_basic(self):
+        request = {
+            'uri': mox.Func(self.saver('uri')),
+            'method': 'POST',
+            'headers': mox.Func(self.saver('headers')),
+            'body': mox.Func(self.saver('body')),
+        }
+        response = {
+            'status': 201,  # created
+        }
+
+        http = typepad.TypePadClient()
+        typepad.client = http
+        http.add_credentials(
+            OAuthConsumer('consumertoken', 'consumersecret'),
+            OAuthToken('tokentoken', 'tokensecret'),
+            domain='api.typepad.com',
+        )
+
+        mock = mox.Mox()
+        mock.StubOutWithMock(http, 'request')
+        http.request(**request).AndReturn((response, ''))
+        mock.ReplayAll()
+
+        asset = typepad.Photo()
+        asset.title = "Fake photo"
+        asset.content = "This is a made-up photo for testing automated browser style upload."
+
+        fileobj = StringIO('hi hello pretend file')
+        brupload = typepad.BrowserUploadEndpoint()
+        brupload.upload(asset, fileobj, "image/png",
+            post_type='photo')
+
+        mock.VerifyAll()
+
+        self.assert_(self.uri)
+        self.assert_(self.uri.startswith('http://api.typepad.com/browser-upload.json'))
+        uriparts = list(urlparse(self.uri))
+        querydict = cgi.parse_qs(uriparts[4])
+
+        # TODO: really verify the signature
+
+        # Verify the headers and body.
+        self.assert_(self.headers)
+        self.assert_(self.body)
+        responsemsg = self.message_from_response(self.headers, self.body)
+
+        content_type = responsemsg.get_content_type()
+        self.assert_(content_type)
+        self.assert_(not responsemsg.defects)
+
+        # Check that the unparsed body has its internal mime headers
+        # separated by full CRLFs, not just LFs.
+        self.assert_('\r\nContent-Type:' in self.body)
+        # Check that boundaries are separated by full CRLFs too.
+        boundary = responsemsg.get_param('boundary')
+        self.assert_(boundary + '\r\n' in self.body)
+
+        # Make sure we're only putting credentials in the query string, not
+        # the headers.
+        self.assert_('oauth_signature' in querydict)
+        self.assert_('authorization' not in responsemsg)
+        self.assert_('Authorization' not in responsemsg)
+
+        bodyparts = responsemsg.get_payload()
+        self.assertEquals(len(bodyparts), 3)
+        bodyparts = dict((part.get_param('name', header='content-disposition'),
+            part) for part in bodyparts)
+
+        self.assertEquals(bodyparts['post_type'].get_payload(), 'photo')
+        self.assert_('redirect_to' not in bodyparts)
+
+        asset_json = bodyparts['asset'].get_payload()
+        self.assert_(json_equals({
+            'title': 'Fake photo',
+            'content': 'This is a made-up photo for testing automated browser style upload.',
+            'objectTypes': ['tag:api.typepad.com,2009:Photo'],
+        }, asset_json))
+
+        filepart = bodyparts['file']
+        self.assertEquals(filepart.get_payload(decode=False), 'hi hello pretend file')
+        filelength = filepart.get('content-length')
+        self.assertEquals(int(filelength), len('hi hello pretend file'))
+        filename = filepart.get_param('filename', header='content-disposition')
+        self.assert_(filename)
+
+    def test_redirect(self):
+        request = {
+            'uri': mox.Func(self.saver('uri')),
+            'method': 'POST',
+            'headers': mox.Func(self.saver('headers')),
+            'body': mox.Func(self.saver('body')),
+        }
+        response = {
+            'status': 302,
+            'location': 'http://client.example.com/hi',
+        }
+
+        http = typepad.TypePadClient()
+        typepad.client = http
+        http.add_credentials(
+            OAuthConsumer('consumertoken', 'consumersecret'),
+            OAuthToken('tokentoken', 'tokensecret'),
+            domain='api.typepad.com',
+        )
+
+        mock = mox.Mox()
+        mock.StubOutWithMock(http, 'request')
+        http.request(**request).AndReturn((response, ''))
+        mock.ReplayAll()
+
+        asset = typepad.Photo()
+        asset.title = "Fake photo"
+        asset.content = "This is a made-up photo for testing automated browser style upload."
+
+        fileobj = StringIO('hi hello pretend file')
+        brupload = typepad.BrowserUploadEndpoint()
+        brupload.upload(asset, fileobj, "image/png",
+            redirect_to='http://client.example.com/hi',
+            post_type='photo')
+
+        mock.VerifyAll()
+
+        # Verify the headers and body.
+        self.assert_(self.headers)
+        self.assert_(self.body)
+        response = self.message_from_response(self.headers, self.body)
+
+        bodyparts = response.get_payload()
+        self.assertEquals(len(bodyparts), 4)
+        bodyparts = dict((part.get_param('name', header='content-disposition'),
+            part) for part in bodyparts)
+
+        # Confirm that the redirect_to was sent.
+        self.assert_('redirect_to' in bodyparts)
+        self.assertEquals(bodyparts['redirect_to'].get_payload(),
+            'http://client.example.com/hi')
+
+    def test_real_file(self):
+        request = {
+            'uri': mox.Func(self.saver('uri')),
+            'method': 'POST',
+            'headers': mox.Func(self.saver('headers')),
+            'body': mox.Func(self.saver('body')),
+        }
+        response = {
+            'status': 201,
+        }
+
+        http = typepad.TypePadClient()
+        typepad.client = http
+        http.add_credentials(
+            OAuthConsumer('consumertoken', 'consumersecret'),
+            OAuthToken('tokentoken', 'tokensecret'),
+            domain='api.typepad.com',
+        )
+
+        mock = mox.Mox()
+        mock.StubOutWithMock(http, 'request')
+        http.request(**request).AndReturn((response, ''))
+        mock.ReplayAll()
+
+        asset = typepad.Photo()
+        asset.title = "One-by-one png"
+        asset.content = "This is a 1&times;1 transparent PNG."
+
+        fileobj = file(os.path.join(os.path.dirname(__file__), 'onebyone.png'))
+        brupload = typepad.BrowserUploadEndpoint()
+        brupload.upload(asset, fileobj, "image/png",
+            post_type='photo')
+
+        mock.VerifyAll()
+
+        response = self.message_from_response(self.headers, self.body)
+
+        (filepart,) = [part for part in response.get_payload()
+            if part.get_param('name', header='content-disposition') == 'file']
+
+        self.assertEquals(filepart.get_content_type(), 'image/png')
+
+        # If there's a transfer encoding, it has to be the identity encoding.
+        transenc = filepart.get('content-transfer-encoding')
+        self.assert_(transenc is None or transenc == 'identity')
+
+        # Check that the file content is equivalent without decoding.
+        fileobj.seek(0)
+        filecontent = fileobj.read()
+        fileobj.close()
+        self.assertEquals(filepart.get_payload(decode=False), filecontent)
+
+        filelength = filepart.get('content-length')
+        self.assertEquals(int(filelength), len(filecontent))
+
+
 if __name__ == '__main__':
     utils.log()
     unittest.main()
