@@ -49,12 +49,15 @@ The module contains:
 from urlparse import urljoin, urlparse, urlunparse
 import cgi
 import inspect
+import sys
 import urllib
 
 from batchhttp.client import BatchError
 import remoteobjects
 from remoteobjects.dataobject import find_by_name
 from remoteobjects.promise import PromiseError
+import remoteobjects.listobject
+import httplib2
 
 import typepad
 from typepad import fields
@@ -65,6 +68,15 @@ classes_by_object_type = {}
 
 class TypePadObjectMetaclass(remoteobjects.RemoteObject.__metaclass__):
 
+    """A metaclass for creating new `TypePadObject` classes.
+
+    In addition to the normal behavior of `RemoteObject` class creation,
+    classes created by `TypePadObjectMetaclass` are classified by their
+    ``object_type`` members, so their instances can be reclassified based on
+    ``object_types`` data in API responses.
+
+    """
+
     def __new__(cls, name, bases, attrs):
         newcls = super(TypePadObjectMetaclass, cls).__new__(cls, name, bases, attrs)
         try:
@@ -81,10 +93,9 @@ class TypePadObject(remoteobjects.RemoteObject):
     """A `RemoteObject` representing an object in the TypePad API.
 
     All HTTP requests made for a `TypePadObject` are made through the
-    `typepad.client` instance of `batchhttp.client.BatchClient`. Unlike other
-    `PromiseObject` instances, `TypePadObject` instances cannot be
-    independently delivered; they must be delivered by an outside object
-    (namely `typepad.client`).
+    `typepad.client` user agent instance. Unlike other `PromiseObject`
+    instances, `TypePadObject` instances cannot be independently delivered;
+    they must be delivered by an outside object (namely `typepad.client`).
 
     """
 
@@ -94,22 +105,15 @@ class TypePadObject(remoteobjects.RemoteObject):
     batch_requests = True
 
     object_types = fields.List(fields.Field(), api_name='objectTypes')
-
-    @classmethod
-    def get_response(cls, url, http=None, **kwargs):
-        """Performs the given HTTP request through `typepad.client`
-        `BatchClient` instance."""
-        if not urlparse(url)[1]:  # network location
-            url = urljoin(typepad.client.endpoint, url)
-        return super(TypePadObject, cls).get_response(url, http=typepad.client, **kwargs)
+    """A list of URIs that identify the type of TypePad content object this
+    is."""
 
     @classmethod
     def get(cls, url, *args, **kwargs):
         """Promises a new `TypePadObject` instance for the named resource.
 
-        If parameter `url` is not an absolute URL but the `TypePadObject`
-        class has been configured with a base URL, the resulting instance will
-        reference the given URL relative to the base address.
+        If parameter `url` is not an absolute URL, the resulting instance will
+        reference the given URL relative to the TypePad API's base address.
 
         If batch requests are enabled, the request that delivers the resulting
         `TypePadObject` instance will be added to the `typepad.client`
@@ -122,17 +126,122 @@ class TypePadObject(remoteobjects.RemoteObject):
         if not urlparse(url)[1]:  # network location
             url = urljoin(typepad.client.endpoint, url)
 
+        kwargs['http'] = typepad.client
+
         ret = super(TypePadObject, cls).get(url, *args, **kwargs)
         ret.batch_requests = kwargs.get('batch', cls.batch_requests)
         if ret.batch_requests:
+            cb = kwargs.get('callback', ret.update_from_response)
             try:
-                typepad.client.batch(ret.get_request(), ret.update_from_response)
+                typepad.client.batch(ret.get_request(), cb)
             except BatchError, ex:
                 # Remember our caller in case we need to complain about
                 # delivery later.
                 ret._origin = inspect.stack()[1][1:4]
         return ret
 
+    def post(self, obj, http=None):
+        """Adds another `TypePadObject` to this remote resource through an HTTP
+        ``POST`` request, as in `HttpObject.post()`.
+
+        Regardless of the `http` parameter, the request is performed with the
+        `typepad.client` user agent.
+
+        """
+        http = typepad.client
+        return super(TypePadObject, self).post(obj, http=http)
+
+    def put(self, http=None):
+        """Saves a previously requested `TypePadObject` back to its remote
+        resource through an HTTP ``PUT`` request, as in `HttpObject.put()`.
+
+        Regardless of the `http` parameter, the request is performed with the
+        `typepad.client` user agent.
+
+        """
+        http = typepad.client
+        return super(TypePadObject, self).put(http=http)
+
+    def delete(self, http=None):
+        """Deletes the remote resource represented by this `TypePadObject`
+        instance through an HTTP ``DELETE`` request.
+
+        Regardless of the `http` parameter, the request is performed with the
+        `typepad.client` user agent.
+
+        """
+        http = typepad.client
+        return super(TypePadObject, self).delete(http=http)
+
+    def head(self, http=None):
+        http = typepad.client
+
+        ret = super(TypePadObject, self).head(http=http)
+        try:
+            typepad.client.batch(ret.get_request(), ret.update_from_response)
+        except BatchError, ex:
+            # Remember our caller in case we need to complain about
+            # delivery later.
+            ret._origin = inspect.stack()[1][1:4]
+        return ret
+
+    def options(self, http=None):
+        http = typepad.client
+
+        ret = super(TypePadObject, self).options(http=http)
+        try:
+            typepad.client.batch(ret.get_request(), ret.update_from_response)
+        except BatchError, ex:
+            # Remember our caller in case we need to complain about
+            # delivery later.
+            ret._origin = inspect.stack()[1][1:4]
+        return ret
+
+    def reclass_for_data(self, data):
+        """Modifies this `TypePadObject` instance to be an instance of the
+        specific `TypePadObject` subclass specified in `data`.
+
+        If the data specify a different `TypePadObject` subclass than the one
+        of which `self` is an instance, `self` will be modified to be an
+        instance of the described class. That is, if:
+
+        * the `data` parameter is a dictionary containing an ``objectTypes``
+          item,
+        * that ``objectTypes`` item is a list of object type identifiers, and
+        * the object type identifiers describe a different `TypePadObject`
+          subclass besides the one `self` belongs to,
+
+        `self` will be turned into an instance of the class specified in the
+        `data` parameter's ``objectTypes`` list.
+
+        This method returns ``True`` if the instance was changed to be a
+        different class, or ``False`` if it was not modified.
+
+        """
+        # What should I be?
+        objtypes = ()
+        try:
+            objtypes = data['objectTypes']
+        except (TypeError, KeyError):
+            pass
+
+        for objtype in objtypes:
+            try:
+                objclsname = classes_by_object_type[objtype]
+                objcls = find_by_name(objclsname)  # KeyError
+            except KeyError:
+                continue
+
+            # Is that a change?
+            if objcls is not self.__class__:
+                self.__class__ = objcls
+
+                # Have update_from_dict() start over.
+                return True
+
+        # We're already that class, so go ahead.
+        return False
+
     def update_from_dict(self, data):
         """Updates this object with the given data, transforming it into an
         instance of a different `TypePadObject` subclass if necessary.
@@ -154,28 +263,13 @@ class TypePadObject(remoteobjects.RemoteObject):
         `self` will be turned into an instance of the class specified in the
         `data` parameter's ``objectTypes`` list.
 
+        Override the `reclass_for_data()` method to change when an instance is
+        modified to be of a different subclass.
+
         """
-        # What should I be?
-        objtypes = ()
-        if not hasattr(self, '_originaldata'):
-            try:
-                objtypes = data['objectTypes']
-            except (TypeError, KeyError):
-                pass
-
-        for objtype in objtypes:
-            try:
-                objclsname = classes_by_object_type[objtype]
-                objcls = find_by_name(objclsname)  # KeyError
-            except KeyError:
-                continue
-
-            # Is that a change?
-            if objcls is not self.__class__:
-                self.__class__ = objcls
-
-                # Redispatch from the beginning.
-                return self.update_from_dict(data)
+        if self.reclass_for_data(data):
+            # Redispatch from the beginning.
+            return self.update_from_dict(data)
 
         super(TypePadObject, self).update_from_dict(data)
 
@@ -188,6 +282,7 @@ class TypePadObject(remoteobjects.RemoteObject):
                 pass
 
     def to_dict(self):
+        """Encodes the `TypePadObject` instance to a dictionary."""
         ret = super(TypePadObject, self).to_dict()
         if 'objectTypes' not in ret and self.object_type is not None:
             ret['objectTypes'] = (self.object_type,)
@@ -224,15 +319,35 @@ class Link(TypePadObject):
     """
 
     rel             = fields.Field()
+    """A keyword representing the relationship type of the link.
+
+    See the Link Relationship Keywords section of the API documentation for
+    possible values and their meanings.
+
+    """
     href            = fields.Field()
-    html            = fields.Field()
+    """The absolute URL of the target resource."""
     type            = fields.Field()
+    """The MIME media type of the target resource."""
     width           = fields.Field()
+    """Where the link is to a visual media item (a photo, for example), the
+    width of the item in pixels."""
     height          = fields.Field()
-    duration        = fields.Field()
+    """Where the link is to a visual media item (a photo, for example), the
+    height of the item in pixels."""
     total           = fields.Field()
-    by_user         = fields.Field(api_name="byUser")
+    """Where the link is to a list resource, the total number of items in that list."""
     allowed_methods = fields.List(fields.Field(), api_name='allowedMethods')
+    """If present, a list of HTTP methods that the TypePad user who requested
+    the containing resource is allowed to perform on the target resource.
+
+    An empty list indicates no methods are allowed. If no list is present, use
+    an ``OPTIONS`` or ``GET`` request to determine the available methods.
+
+    """
+    html            = fields.Field()
+    duration        = fields.Field()
+    by_user         = fields.Field(api_name="byUser")
 
     def __repr__(self):
         """Returns a developer-readable representation of this object."""
@@ -259,6 +374,19 @@ class LinkSet(set, TypePadObject):
         instances contained in this `LinkSet`."""
         return sorted([x.to_dict() for x in self])
 
+    def __contains__(self, key):
+        """Returns a boolean result for whether the `LinkSet` contains
+        a `Link` matching the key, as compared to the 'rel' member of
+        each `Link`.
+
+        """
+
+        for x in self:
+            if x.rel == key:
+                return True
+
+        return False
+
     def __getitem__(self, key):
         """Returns the `Link` or `LinkSet` described by the given key.
 
@@ -289,6 +417,9 @@ class LinkSet(set, TypePadObject):
         elif key.startswith('width__'):
             width = int(key[7:])
             return self.link_by_width(width)
+        elif key.startswith('size__'):
+            size = int(key[7:])
+            return self.link_by_size(size)
         elif key.startswith('maxwidth__'):
             width = int(key[10:])
             links_by_width = dict([(x.width, x) for x in self if x.width <= width])
@@ -315,6 +446,7 @@ class LinkSet(set, TypePadObject):
         If there are no images in this `LinkSet`, returns `None`.
 
         """
+
         if width == 0:
             # Select the widest link
             better = lambda x: best is None or best.width < x.width
@@ -333,102 +465,50 @@ class LinkSet(set, TypePadObject):
 
         return best
 
+    def link_by_size(self, size=0):
+        """Returns the `Link` instance from this `LinkSet` that best matches
+        the requested display size.
 
-class SequenceProxy(object):
+        If optional parameter `size` is specified, the `Link` instance
+        representing the smallest image bigger or matching the `size`
+        specified is returned. If there is no such image, or if `size` is
+        not specified, the `Link` for the biggest image is returned.
 
-    """An abstract class implementing the sequence protocol by proxying it to
-    an instance attribute.
-
-    `SequenceProxy` instances act like sequences by forwarding all sequence
-    method calls to their `entries` attributes. The `entries` attribute should
-    be a list or some other that implements the sequence protocol.
-
-    """
-
-    def make_sequence_method(methodname):
-        """Makes a new function that proxies calls to `methodname` to the
-        `entries` attribute of the instance on which the function is called as
-        an instance method."""
-        def seqmethod(self, *args, **kwargs):
-            # Proxy these methods to self.entries.
-            return getattr(self.entries, methodname)(*args, **kwargs)
-        seqmethod.__name__ = methodname
-        return seqmethod
-
-    __len__      = make_sequence_method('__len__')
-    __getitem__  = make_sequence_method('__getitem__')
-    __setitem__  = make_sequence_method('__setitem__')
-    __delitem__  = make_sequence_method('__delitem__')
-    __iter__     = make_sequence_method('__iter__')
-    __reversed__ = make_sequence_method('__reversed__')
-    __contains__ = make_sequence_method('__contains__')
-
-
-class ListOf(TypePadObjectMetaclass, remoteobjects.ListObject.__metaclass__):
-
-    """Metaclass defining a `ListObject` containing a list of some other
-    class's instances.
-
-    Unlike most metaclasses, this metaclass can be called directly to define
-    new `ListObject` classes that contain objects of a specified other class,
-    like so:
-
-    >>> ListOfEntry = ListOf(Entry)
-
-    This is equivalent to defining ``ListOfEntry`` yourself:
-
-    >>> class ListOfEntry(typepad.ListObject):
-    ...     entryclass = Entry
-
-    which is a `ListObject` of ``Entry`` instances.
-
-    """
-
-    _subclasses = {}
-
-    def __new__(cls, name, bases=None, attr=None):
-        """Creates a new `ListObject` subclass.
-
-        If `bases` and `attr` are specified, as in a regular subclass
-        declaration, a new `ListObject` subclass bound to no particular
-        `TypePadObject` class is created. `name` is the declared name of the
-        new class, as usual.
-
-        If only `name` is specified, that value is used as a reference to a
-        `TypePadObject` class to which the new `ListObject` class is bound.
-        The `name` parameter can be either a name or a `TypePadObject` class,
-        as when declaring a `fields.Object` on a class.
+        If there are no images in this `LinkSet`, returns `None`.
 
         """
-        direct = attr is None
-        if direct:
-            # Don't bother making a new subclass if we already made one for
-            # this target.
-            if name in cls._subclasses:
-                return cls._subclasses[name]
 
-            entryclass = name
-            if callable(entryclass):
-                name = cls.__name__ + entryclass.__name__
+        # highest resolution image that still fits in bounding box
+        fits = None
+        # smallest resolution image that is bigger than bounding box
+        oversize = None
+        # highest resolution image
+        original = None
+
+        for image in self:
+            # logic for assigning 'oversize'
+            if image.width > size or image.height > size:
+                if oversize is None or (oversize.width > image.width or oversize.height > image.height):
+                    oversize = image
             else:
-                name = cls.__name__ + entryclass
-            bases = (ListObject,)
-            attr = {'entryclass': entryclass}
+                # logic for assigning 'fits'
+                if fits is None or (fits.width < image.width or fits.height < image.height):
+                    fits = image
+            if original is None or (original.width < image.width or original.height < image.height):
+                original = image
+
+        if size == 0:
+            return original
         else:
-            # Make sure classes we create conventionally are SequenceProxies.
-            # As that includes ListObject, classes that are created directly
-            # inherit SequenceProxy behavior through ListObject.
-            bases = bases + (SequenceProxy,)
-
-        newcls = super(ListOf, cls).__new__(cls, name, bases, attr)
-        # Save the result for later direct invocations.
-        if direct:
-            orig_name = attr['entryclass']
-            cls._subclasses[orig_name] = newcls
-        return newcls
+            return fits or oversize or original
 
 
-class ListObject(TypePadObject, remoteobjects.ListObject):
+class ListOf(remoteobjects.listobject.PageOf, TypePadObjectMetaclass):
+
+    _modulename = 'typepad.tpobject._lists'
+
+
+class ListObject(TypePadObject, remoteobjects.PageObject):
 
     """A `TypePadObject` representing a list of other `TypePadObject`
     instances.
@@ -454,31 +534,37 @@ class ListObject(TypePadObject, remoteobjects.ListObject):
     __metaclass__ = ListOf
 
     total_results = fields.Field(api_name='totalResults')
+    """The total number of items in the overall list resource (of which this
+    `ListObject` instance may be only a segment)."""
     start_index   = fields.Field(api_name='startIndex')
+    """The index in the overall list resource of the first item in this
+    `ListObject` instance.
+
+    The first item in the list has index 1.
+
+    """
     links         = fields.Object('LinkSet')
+    """A `LinkSet` of links related to this list resource."""
     entries       = fields.List(fields.Field())
+    """A list of items in this list resource."""
 
     filterorder = ['following', 'follower', 'friend', 'nonreciprocal',
         'published', 'unpublished', 'spam', 'admin', 'member',
         'by-group', 'by-user', 'photo', 'post', 'video', 'audio', 'comment',
         'link']
 
-    def to_dict(self):
-        d = super(ListObject, self).to_dict()
-        d['entries'] = [x.to_dict() for x in self.entries]
-        return d
-
     def count(self):
+        """Returns the number of items in the overall list resource, of which
+        this `ListObject` instance may be only a segment."""
         return int(self.total_results)
 
     def filter(self, **kwargs):
         """Returns a new `ListObject` instance representing the same endpoint
         as this `ListObject` instance with the additional filtering applied.
 
-        This method filters the `ListObject` as does
-        `remoteobjects.ListObject.filter()`, but specially treats filters
-        defined in the TypePad API. These special filters are not added in as
-        query parameters but as path components.
+        This method filters the `ListObject` as does `RemoteObject.filter()`,
+        but specially treats filters defined in the TypePad API. These special
+        filters are not added in as query parameters but as path components.
 
         """
         # Split the list's URL into URL parts, filters, and queryargs.
@@ -506,13 +592,18 @@ class ListObject(TypePadObject, remoteobjects.ListObject):
 
         # Add kwargs into the filters and queryargs as appropriate.
         for k, v in kwargs.iteritems():
+            # ignore this kwarg
+            if k in ('callback', 'batch'):
+                continue
+
             # handle case where value is a TypePadObject. If it is, check for
             # 'url_id' and if present, use that. If not, raise an exception
             if isinstance(v, typepad.api.TypePadObject):
                 if hasattr(v, 'url_id'):
                     v = v.url_id
                 else:
-                    raise Exception("invalid object filter value for parameter %k; object must have a url_id property to filter by object" % k)
+                    raise ValueError("""invalid object filter value for parameter %s; """
+                        """object must have a url_id property to filter by object""" % k)
             # Convert by_group to by-group.
             k = k.replace('_', '-')
             # Convert by_group=7 to by_group='7'.
@@ -537,9 +628,12 @@ class ListObject(TypePadObject, remoteobjects.ListObject):
         parts[4] = urllib.urlencode(queryargs)
         newurl = urlunparse(parts)
 
-        ret = self.get(newurl)
-        ret.of_cls = self.of_cls
-        return ret
+        getargs = {}
+        if 'callback' in kwargs:
+            getargs['callback'] = kwargs['callback']
+        if 'batch' in kwargs:
+            getargs['batch'] = kwargs['batch']
+        return self.get(newurl, **getargs)
 
     def __getitem__(self, key):
         """Returns the specified members of the `ListObject` instance's
@@ -556,17 +650,3 @@ class ListObject(TypePadObject, remoteobjects.ListObject):
             args['max_results'] = key.stop
         return self.filter(**args)
 
-    def update_from_dict(self, data):
-        """Fills this `ListObject` instance with the data from the given
-        dictionary.
-
-        The contents of the dictionary's `entries` member will be decoded into
-        instances of this `ListObject` instance's entry class.
-
-        """
-        super(ListObject, self).update_from_dict(data)
-        # Post-convert all the "entries" list items to our entry class.
-        entryclass = self.entryclass
-        if not callable(entryclass):
-            entryclass = find_by_name(entryclass)
-        self.entries = [entryclass.from_dict(d) for d in self.entries if d is not None]
